@@ -92,8 +92,13 @@ PROPERTY_LINE_REPLACEMENT_VALUE = "    \g<1>: \g<2>"
 """ The property line replacement value """
 
 RULES_MAP = dict(
-    margin = ("zero_to_multiple",),
-    border = ("zero_to_none",)
+    left = ("zero_to_zero_px",),
+    top = ("zero_to_zero_px",),
+    right = ("zero_to_zero_px",),
+    bottom = ("zero_to_zero_px",),
+    border = ("zero_to_none",),
+    margin = ("zero_to_multiple", "two_to_multiple"),
+    padding = ("zero_to_multiple", "two_to_multiple")
 )
 """ The map that associates various property names
 with the various rules that should be applied to them """
@@ -154,7 +159,7 @@ def get_property_index(property_line, property_order, line_number):
     property_index = property_order.index(property_name)
     return property_index
 
-def write_lines(output_buffer, lines, windows_newline):
+def write_lines(output_buffer, lines, windows_newline = True, avoid_empty = False):
     """
     Writes the provided lines to the output buffer, considering the windows
     newline option.
@@ -165,14 +170,21 @@ def write_lines(output_buffer, lines, windows_newline):
     @param lines: The list of lines to output.
     @type windows_newline: bool
     @param windows_newline: If the windows newline should be used.
+    @type avoid_empty: bool
+    @param avoid_empty: If the empty lines (after strip operation)
+    should be considered invalid and not written to buffer.
     """
 
-    # for each line
-    for line in lines:
-        # writes the line, considering the windows newline
-        write_line(output_buffer, line, windows_newline)
+    # iterates over the complete set of lines that are going
+    # to be written to the buffer and writes their value
+    for line in lines: write_line(
+        output_buffer,
+        line,
+        windows_newline = windows_newline,
+        avoid_empty = avoid_empty
+    )
 
-def write_line(output_buffer, line, windows_newline):
+def write_line(output_buffer, line, windows_newline = True, avoid_empty = False):
     """
     Writes the provided line to the output buffer, considering the windows
     newline option. This is considered a normalization operation.
@@ -183,6 +195,9 @@ def write_line(output_buffer, line, windows_newline):
     @param lines: The list of lines to output.
     @type windows_newline: bool
     @param windows_newline: If the windows newline should be used.
+    @type avoid_empty: bool
+    @param avoid_empty: If the empty lines (after strip operation)
+    should be considered invalid and not written to buffer.
     """
 
     # right strips the line so that no extra space characters
@@ -191,27 +206,37 @@ def write_line(output_buffer, line, windows_newline):
     line = line.rstrip()
     output_buffer.write(line)
 
+    # in case the avoid empty flag is set and the line value
+    # is considered invalid/empty, it's ignored
+    if avoid_empty and not line: return
+
     # in case the newline mode is of type windows writes the
     # typical carriage return new line values otherwise writes
     # the unix simpler value with just a newline character
     if windows_newline: output_buffer.write("\r\n")
     else: output_buffer.write("\n")
 
-def process_property_lines(property_lines, line_number):
+def process_property_lines(property_lines, line_number, avoid_empty = False):
     """
-    Processes the property lines.
+    Processes the property lines, this process should be
+    iterative for each of the lines passed in the list.
 
     @type property_lines: List
     @param property_lines: The property lines to process.
     @type line_number: int
     @param line_number: The approximate line number for this processing.
+    @type avoid_empty: bool
+    @param avoid_empty: If the empty lines should avoid any
+    kind of processing (extra errors may occur).
     @rtype: List
     @return: The processed property lines.
     """
 
-    # processes the property lines
+    # processes the property lines one by one so that the
+    # properly represent the same semantic value but with
+    # a better/standard representation of the value
     processed_property_lines = [process_property_line(property_line, line_number)\
-        for property_line in property_lines]
+        for property_line in property_lines if not avoid_empty or property_line.strip()]
 
     # returns the processed property lines
     return processed_property_lines
@@ -254,6 +279,10 @@ def process_property_line(property_line, line_number):
     # returns the processed property line
     return property_line
 
+def rule_zero_to_zero_px(name, value):
+    if not value in ("0",): return
+    return "%s: 0px;" % name
+
 def rule_zero_to_multiple(name, value):
     if not value in ("0", "0px"): return
     return "%s: 0px 0px 0px 0px;" % name
@@ -262,22 +291,41 @@ def rule_zero_to_none(name, value):
     if not value in ("0", "0px"): return
     return "%s: none;" % name
 
+def rule_two_to_multiple(name, value):
+    parts = value.split()
+    if not len(parts) == 2: return
+    return "%s: %s %s;" % (name, value, value)
+
 def process_rules(property_line, line_number):
-    # unpacks the property line into name and value so that proper
-    # per name rule application is possible for the line
-    property_name, property_value = property_line.split(":", 1)
+    # retrieves the initial value for the property name
+    # by splitting the line value, this value is going to
+    # be used for the resolution of the final rules (set)
+    property_name, _property_value = property_line.split(":", 1)
     property_name = property_name.strip()
-    property_value = property_value.strip()
-    property_value = property_value.rstrip(";")
 
     # tries to retrieve the set of rules (as strings) that are
     # going to be applied to the current property and then runs
     # the iteration cycle to apply each of them on the line
     rules = RULES_MAP.get(property_name, [])
     for rule in rules:
+        # creates the (rule) method name by appending the name of
+        # the current rule to the prefix and tries to retrieve a
+        # global function with such name in case it does not exists
+        # continues the current loop (nothing to be done)
         method_name = "rule_" + rule
         method = globals().get(method_name, None)
         if not method: continue
+
+        # unpacks the property line into name and value so that proper
+        # per name rule application is possible for the line
+        property_name, property_value = property_line.split(":", 1)
+        property_name = property_name.strip()
+        property_value = property_value.strip()
+        property_value = property_value.rstrip(";")
+
+        # calls the proper rule method with the (current) property name
+        # and value and verifies if a valid result is returned in case
+        # it does replaces the property line value with the new value
         result = method(property_name, property_value)
         if not result: continue
         property_line = result
@@ -356,7 +404,13 @@ def skip_rule(start_line, rules_skip):
     # returns the skip rule value
     return skip_rule
 
-def cleanup_properties(input_buffer, windows_newline, fix_extra_newlines, property_order, rules_skip):
+def cleanup_properties(
+    input_buffer,
+    windows_newline = True,
+    fix_extra_newlines = True,
+    property_order = (),
+    rules_skip = ()
+):
     """
     Cleans up the property lines. Sorts the css properties in the file,
     by the specified property order.
@@ -474,15 +528,21 @@ def cleanup_properties(input_buffer, windows_newline, fix_extra_newlines, proper
 
                 # sorts the various property lines and the processes them
                 property_lines = sorted(rule_lines, key = get_comparison_key)
-                property_lines = process_property_lines(property_lines, line_number)
+                property_lines = process_property_lines(
+                    property_lines,
+                    line_number,
+                    avoid_empty = fix_extra_newlines
+                )
 
                 # writes the lines to the buffer, considering the windows newline
                 # and then writes the line
-                write_lines(output_buffer, property_lines, windows_newline)
-                write_line(output_buffer, line, windows_newline)
-
-                # writes the final newline
-                fix_extra_newlines and write_line(output_buffer, "\n", windows_newline)
+                write_lines(
+                    output_buffer,
+                    property_lines,
+                    windows_newline = windows_newline,
+                    avoid_empty = fix_extra_newlines
+                )
+                write_line(output_buffer, line, windows_newline = windows_newline)
 
                 # resets the newlines counter and then
                 # enables the needs newline flag
@@ -493,16 +553,9 @@ def cleanup_properties(input_buffer, windows_newline, fix_extra_newlines, proper
                 continue
         # in case this line is part of a valid rule set
         elif rule_started:
-            if fix_extra_newlines:
-                # strips the line and then appends the line
-                # to the rule set in case it's not a newline
-                line_stripped = line.strip()
-                line_stripped and rule_lines.append(line)
-            else:
-                # appends the line to the rule set
-                rule_lines.append(line)
-
+            # appends the line to the rule set, and then
             # skips outputting the line to the buffer
+            rule_lines.append(line)
             continue
         # in case this is part of rule selector declaration
         elif "," in line:
@@ -513,14 +566,11 @@ def cleanup_properties(input_buffer, windows_newline, fix_extra_newlines, proper
             # increments the newlines count
             newlines += 1
 
-            # in case the fix extra newlines flag is active
-            if fix_extra_newlines:
-                # skip processing
-                continue
-            # otherwise in case this is an extra newline
-            elif not needs_newline and newlines > 1:
-                # logs a warning about this extra newline
-                extra.warn("Found extra newline at line %d" % line_number)
+            # otherwise in case this is an extra newline, must either
+            # remove it or print a warning message depending on mode
+            if not needs_newline and newlines > 1:
+                if fix_extra_newlines: continue
+                else: extra.warn("Found extra newline at line %d" % line_number)
 
             # disables the needs newline flag
             needs_newline = False
@@ -544,7 +594,10 @@ def cleanup_properties(input_buffer, windows_newline, fix_extra_newlines, proper
     number_lines = output_buffer_value.count("\n")
 
     if not number_lines == number_original_lines and not fix_extra_newlines:
-        raise Exception("number of lines in processed file (%d) is different from original file (%d)" % (number_lines, number_original_lines))
+        raise Exception(
+            "number of lines in processed file (%d) is different from original file (%d)" %\
+            (number_lines, number_original_lines)
+        )
 
     return output_buffer
 
@@ -580,10 +633,10 @@ def cleanup_stylesheets(
         # complete set of rules and clean the file
         string_buffer = cleanup_properties(
             file,
-            windows_newline,
-            fix_extra_newlines,
-            property_order,
-            rules_skip
+            windows_newline = windows_newline,
+            fix_extra_newlines = fix_extra_newlines,
+            property_order = property_order,
+            rules_skip = rules_skip
         )
 
         # retrieves the string value from the output
@@ -805,5 +858,9 @@ def main():
                 rules_skip
             )
 
+    # verifies if there were messages printed to the standard
+    # error output and if that's the case exits in error
+    sys.exit(1 if extra.has_errors() else 0)
+
 if __name__ == "__main__":
     main()
